import React, { Component } from "react";
import { NavigationContainer } from "@react-navigation/native";
import { createStackNavigator } from "@react-navigation/stack";
import PI from "./PI";
import ThankYou from "./src/Screens/ThankYou";
import Header from "./src/Screens/Header";
import VideoRecorder from "./src/common/VideoRecorder";
import VideoPreview from "./src/common/VideoPreview";
import DocumentUpload from "./src/common/DocumentUpload";
import { Provider as PaperProvider, DefaultTheme } from "react-native-paper";
import { navigationRef } from "./src/common/RootNavigation";
import { Provider } from "react-redux";
import store from "./src/redux/store";
import Progress from "./src/Screens/Progress";
// import Tutorial from "./src/Screens/Tutorial";
import Camera from "./src/Screens/Camera";
import PreviewImage from "./src/Screens/PreviewImage";
import Orientation from "../../node_modules/react-native-orientation";
import Toturial from "./src/Screens/Toturial";
import InCorrectImages from "./src/Screens/InCorrectImages";
// Localization
const { setI18nConfig, translate } = require("./src/common/localize/i18n");
import * as RNLocalize from "react-native-localize";

class App extends Component {
  constructor(props) {
    super(props);
    setI18nConfig();
  }
  componentDidMount() {
    Orientation.lockToPortrait();
    RNLocalize.addEventListener("change", this.handleLocalizationChange);
  }
  componentWillUnmount() {
    RNLocalize.removeEventListener("change", this.handleLocalizationChange);
  }

  handleLocalizationChange = () => {
    this.forceUpdate();
    setI18nConfig();
  };
  render() {
    const Stack = createStackNavigator();
    return (
      <NavigationContainer
        ref={navigationRef}
        theme={DefaultTheme}
        independent={true}
      >
        <PaperProvider theme={DefaultTheme}>
          <Provider store={store}>
            <Stack.Navigator screenOptions={{ headerShown: false }}>
              <Stack.Screen
                name="PI"
                component={PI}
                initialParams={this.props}
              />
              <Stack.Screen name="Toturial" component={Toturial} />
              <Stack.Screen name="Progress" component={Progress} />
              <Stack.Screen
                name="ThankYou"
                component={ThankYou}
                initialParams={this.props}
              />
              <Stack.Screen
                name="InCorrectImages"
                component={InCorrectImages}
              />
              <Stack.Screen name="Camera" component={Camera} />
              <Stack.Screen
                name="PreviewImage"
                component={PreviewImage}
                initialParams={this.props}
              />

              {/* <Stack.Screen name="Tutorial" component={Tutorial} />
               /> */}
            </Stack.Navigator>
          </Provider>
        </PaperProvider>
      </NavigationContainer>
    );
  }
}

export default App;
