import React, { useEffect, useRef, useState } from "react";
import { wait } from "../../../../Components/common/commonFunctions";
import {
  StyleSheet,
  View,
  Image,
  Text,
  StatusBar,
  Alert,
  NativeModules,
  DeviceEventEmitter,
  PermissionsAndroid,
} from "react-native";
import {
  SCREEN_HEIGHT,
  SCREEN_WIDTH,
  image_pose_blur,
  image_pose_dark,
  image_pose_wrong,
  VIDEO_TITLE_HN,
  VIDEO_TITLE,
  RC_BOOK,
  RC_BOOK_HN,
  SELFI_IMAGE_HN,
  SELFI_IMAGE,
} from "../common/Constants";
import { translate } from "react-native-digitcamera/src/common/localize/i18n";
import Orientation from "../../../react-native-orientation";
import Geolocation from "@react-native-community/geolocation";
import { TouchableOpacity } from "react-native-gesture-handler";
import RNFetchBlob from "rn-fetch-blob";
import { SELFSURVEYSERVICE_DOCUMENT_URL } from "../Api/ApiUrls";
import ViewShot, { captureRef } from "react-native-view-shot";
import VideoPlayer from "react-native-video-player";
import { poseDetection } from "../common/Services/poseDetection";
import reactotron from "reactotron-react-native";

import {
  CHASSIS_NUMBER,
  ACTIVITY_TYPE_MOTOR_2W_PRE_INSPECTION_SELF_SURVEY,
  ACTIVITY_TYPE_MOTOR_4W_OD_NB_PRE_INSPECTION_SELF_SURVEY,
  ACTIVITY_TYPE_MOTOR_PRE_INSPECTION_SELF_SURVEY,
} from "../common/Constants";
import { chassiesPoseDetetction } from "../common/Services/chassisPoseDetection";
import { useSelector, useDispatch } from "react-redux";
//Global variables
let isBlur = true,
  isDark = true;
var sensorImageDetails = [];
var SensorData;
var mSensorManager = NativeModules.SensorManager;
var accelerometerData = [];
var gyroscopeData = [];
var magnetometerData = [];
var pedometerData = [];
var reTryDataLen = 0;
var message, watchID;
const PreviewImage = ({ navigation, route }) => {
  const imageref = useRef();
  const [dateAndTime, SetDateAndTime] = useState("");
  const [imageName, SetImageName] = useState("");
  const [finalImagePath, SetFinalImagePath] = useState("");
  const [uploadState, SetUploadState] = useState("");
  const dispatch = useDispatch();
  const RetryImageData = useSelector((state) => state.RetryImageData);
  const RetryCountData = useSelector((state) => state.RetryCountData);
  const UploadImageData = useSelector((state) => state.UploadImageData);

  const [locationStatus, setLocationStatus] = useState("");
  const [currentLongitude, setCurrentLongitude] = useState("");
  const [currentLatitude, setCurrentLatitude] = useState("");
  const {
    ImageData,
    ext,
    DocumentImageName,
    activityId,
    activityTypeId,
    webUserId,
    title,
    titleHn,
    subTitle,
    subTitleHn,
    totalSteps,
    stepStencilAssests,
    points,
    image,
    isLast,
  } = route.params;

  const retake = require("../common/Image/retake.png");
  const proceed = require("../common/Image/proceed.png");

  useEffect(() => {
    getTime();
    Orientation.lockToLandscapeLeft();
    startCapturingSensorData();

    const requestLocationPermission = async () => {
      if (Platform.OS === "ios") {
        getOneTimeLocation();
        subscribeLocationLocation();
      } else {
        try {
          const granted = await PermissionsAndroid.request(
            PermissionsAndroid.PERMISSIONS.ACCESS_FINE_LOCATION,
            {
              title: "Location Access Required",
              message: "This App needs to Access your location",
            }
          );
          if (granted === PermissionsAndroid.RESULTS.GRANTED) {
            //To Check, If Permission is granted
            getOneTimeLocation();
            subscribeLocationLocation();
          } else {
            setLocationStatus("Permission Denied");
          }
        } catch (err) {
          console.warn(err);
        }
      }
    };
    requestLocationPermission();
    return () => {
      Geolocation.clearWatch(watchID);
    };
  }, []);

  const subscribeLocationLocation = () => {
    watchID = Geolocation.watchPosition(
      (position) => {
        //Will give you the location on location change

        setLocationStatus("You are Here");

        //getting the Longitude from the location json
        const currentLongitude = JSON.stringify(position.coords.longitude);

        //getting the Latitude from the location json
        const currentLatitude = JSON.stringify(position.coords.latitude);

        //Setting Longitude state
        setCurrentLongitude(currentLongitude);

        //Setting Latitude state
        setCurrentLatitude(currentLatitude);
      },
      (error) => {
        setLocationStatus(error.message);
      },
      {
        enableHighAccuracy: false,
        maximumAge: 1000,
      }
    );
  };

  const getOneTimeLocation = () => {
    setLocationStatus("Getting Location ...");
    Geolocation.getCurrentPosition(
      //Will give you the current location
      (position) => {
        setLocationStatus("You are Here");

        //getting the Longitude from the location json
        const currentLongitude = JSON.stringify(position.coords.longitude);

        //getting the Latitude from the location json
        const currentLatitude = JSON.stringify(position.coords.latitude);

        //Setting Longitude state
        setCurrentLongitude(currentLongitude);

        //Setting Longitude state
        setCurrentLatitude(currentLatitude);
      },
      (error) => {
        setLocationStatus(error.message);
      },
      {
        enableHighAccuracy: false,
        timeout: 30000,
        maximumAge: 1000,
      }
    );
  };

  const getTime = () => {
    const date = new Date().getDate();
    const month = new Date().getMonth() + 1;
    const year = new Date().getFullYear();

    //Time calulation for the display on preciew Screen
    const hour = new Date().getHours();
    const minute = new Date().getMinutes();
    const seconds = new Date().getSeconds();
    SetDateAndTime(
      date +
        "/" +
        month +
        "/" +
        year +
        " - " +
        hour +
        ":" +
        minute +
        ":" +
        seconds
    );
  };
  const startCapturingSensorData = () => {
    //Accelerometer
    mSensorManager.startAccelerometer(100); // To start the accelerometer with a minimum delay of 100ms between events.
    DeviceEventEmitter.addListener("Accelerometer", function (data) {
      accelerometerData.push(data);
    });

    //Gyroscope
    mSensorManager.startGyroscope(100);
    DeviceEventEmitter.addListener("Gyroscope", function (data) {
      gyroscopeData.push(data);
    });

    //Magnetometer
    mSensorManager.startMagnetometer(100);
    DeviceEventEmitter.addListener("Magnetometer", function (data) {
      magnetometerData.push(data);
    });

    //Pedometer
    mSensorManager.startStepCounter(1000);
    DeviceEventEmitter.addListener("StepCounter", function (data) {
      pedometerData.push(data);
    });
  };
  const genrateSensorFile = () => {
    const date = new Date().getDate();
    const month = new Date().getMonth() + 1;
    const year = new Date().getFullYear();

    //Time calulation for the display on preciew Screen
    const hour = new Date().getHours();
    const minute = new Date().getMinutes();
    const seconds = new Date().getSeconds();

    SensorData = {
      activity_id: activityId,
      activity_type: activityTypeId,
      date_time:
        date +
        " " +
        month +
        " " +
        year +
        "  " +
        hour +
        ":" +
        minute +
        ":" +
        seconds,
      imageDetails: sensorImageDetails,
      policy_no: "D042578254", // this need to make dynamic
      reg_no: "GJ05JN0394",
      sensor_data: {
        accelerometers: accelerometerData,
        compass: [],
        gyroscope: gyroscopeData,
        magnetometers: magnetometerData,
        millis: new Date().getTime(),
        pedometer: pedometerData,
      },
    };
    const filename = activityId + "_" + "Sensor.json";
    const destPath = RNFetchBlob.fs.dirs.DownloadDir + "/" + filename;
    RNFetchBlob.fs
      .writeFile(destPath, JSON.stringify(SensorData, null, "  "), "utf8")
      .then((data) => {
        console.log(
          "FILE CREATED AT > ",
          RNFetchBlob.fs.dirs.DownloadDir + "/" + filename
        );
      });
    uploadImage(destPath);

    // stop capture data once all photo completed
    mSensorManager.stopAccelerometer();
    mSensorManager.stopStepCounter();
    mSensorManager.stopMagnetometer();
    mSensorManager.stopGyroscope();

    //  navigate to thank you screen
    navigation.navigate("ThankYou");
  };
  // Navigation functions
  const navigationFunc = (uri) => {
    console.log("Is last in navigation func ", isLast, reTryDataLen);
    console.log("Retry data ", JSON.stringify(RetryImageData, null, "  "));

    if (isLast && reTryDataLen == 0) {
      console.log("FLOW COMPLETED AND IMAGE UPLOADED");
      genrateSensorFile();
    } else {
      if (isLast) {
        console.log(
          "INCORRECT IMAGES TRIGGERD ON LAST IMAGE IS LAST > ",
          isLast
        );
        navigation.navigate("InCorrectImages");
      } else {
        console.log("IMAGE PENDING TO CAPTURE > ");
        if (title == RC_BOOK || titleHn == RC_BOOK_HN) {
          navigation.navigate("PI", { photo: uri, isRCBook: true });
        } else {
          navigation.navigate("PI");
        }
      }
    }
  };
  //CALL CHASSIS POSE DETECTION
  const callChassisPoseDetection = async (uri) => {
    var formdata = new FormData();
    formdata.append("file", {
      uri: uri,
      name: activityId + "_" + DocumentImageName + ext,
      fileName: activityId + "_" + DocumentImageName,
      type: "image/jpeg",
    });
    formdata.append("activityId", activityId);

    const ChassisPoseDetectionData = await chassiesPoseDetetction({
      body: formdata,
    });
    if (
      ChassisPoseDetectionData?.match_ratio >
      ChassisPoseDetectionData?.threshold_match_ratio
    ) {
      uploadImage(uri);
      dispatch({
        type: "REMOVE_BY_TITLE",
        payload: title,
      });
      // await wait(500);
      navigationFunc(uri);
    } else {
      if (RetryCountData.retryCountInfo.retryCount >= 2) {
        //upload and navigate to thank you screen
        uploadImage(uri);
        // navigation.navigate("  ")
        dispatch({
          type: "REMOVE_BY_TITLE",
          payload: title,
        });
        // await wait(500);
        navigationFunc(uri);
      } else {
        //add data to redux
        dispatch({
          type: "RETRY_IMAGES",
          payload: {
            retryImageInfo: {
              imageName: DocumentImageName,
              imagePath: uri,
              fileNameWithExt: activityId + "_" + DocumentImageName + ext,
              fileName: activityId + "_" + DocumentImageName,
              title: title,
              subTitle: subTitle,
              titleHn: titleHn,
              subTitleHn: subTitleHn,
              msg: message,
              stepStencilAssests,
              points,
              image,
            },
          },
        });
        // await wait(500);
        console.log("CHASSIS => count Not >2 ");
        navigationFunc(uri);
      }
    }
  };
  // POSE DETECTION API CALL
  const callPoseDetection = async (uri) => {
    var formdata = new FormData();
    formdata.append("file", {
      uri: uri,
      name: activityId + "_" + DocumentImageName + ext,
      fileName: activityId + "_" + DocumentImageName,
      type: "image/jpeg",
    }),
      formdata.append("policyNumber", "MH12EX1234"),
      formdata.append("activityTypeId", activityTypeId),
      formdata.append("checkQuality", true),
      formdata.append("source", "DIGITEDGE");

    const poseDetectionData = await poseDetection({
      body: formdata,
    });
    isBlur = poseDetectionData?.image_pose_array[0]?.is_blurred;
    isDark = poseDetectionData?.image_pose_array[0]?.is_dark;

    console.log("isBlur", isBlur);
    console.log("isDark", isDark);
    if (isBlur && isDark) {
      message = image_pose_blur + " " + image_pose_dark;
    } else {
      if (isDark) message = image_pose_dark;
      if (isBlur) message = image_pose_blur;
      if (!isDark && !isBlur) message = image_pose_wrong;
    }

    if (!isBlur && !isDark) {
      //if image is good then excute this block
      // upload image
      uploadImage(uri);

      //create sensor data ==> if image is uploaded
      sensorImageDetails.push({
        attemptCount: Number(RetryCountData.retryCountInfo.retryCount) + 1,
        fileName: RetryImageData.retryImageInfo.fileNameWithExt,
        latitude: "21.174533",
        longitude: "72.8571249",
        pose: DocumentImageName,
        success: true,
        timestamp: new Date().getTime(),
      });
      navigationFunc(uri);
    } else {
      //create sensor data ==> if image not  uploaded
      sensorImageDetails.push({
        attemptCount: Number(RetryCountData.retryCountInfo.retryCount) + 1,
        fileName: RetryImageData.retryImageInfo.fileNameWithExt,
        latitude: "21.174533",
        longitude: "72.8571249",
        pose: DocumentImageName,
        success: false,
        timestamp: new Date().getTime(),
      });
      // GO FOR RE-TRY
      //save the details to redux
      if (RetryCountData.retryCountInfo.retryCount >= 2) {
        // upload image without pose detection
        uploadImage(uri);
        navigationFunc(uri);
      } else {
        // try two time to upload image
        dispatch({
          type: "RETRY_IMAGES",
          payload: {
            retryImageInfo: {
              imageName: DocumentImageName,
              imagePath: uri,
              fileNameWithExt: activityId + "_" + DocumentImageName + ext,
              fileName: activityId + "_" + DocumentImageName,
              title: title,
              subTitle: subTitle,
              titleHn: titleHn,
              subTitleHn: subTitleHn,
              msg: message,
              stepStencilAssests,
              points,
              image,
            },
          },
        });
      }
      navigationFunc(uri); //   // for navigate to screen
    }
  };
  //upload images
  const uploadImage = (uri) => {
    console.log("uri", uri);
    const documentUploadData = {
      activityId: activityId,
      documentSource: "DigitEdge",
      geoLocation: {
        deviceIMEI: "",
        deviceName: "",
        deviceTime: "",
        documentSource: "DigitEdge",
        ipAddress: "",
        isBlocked: false,
        latitude: "",
        longitude: "",
        macAddress: "",
        operatingSystem: "",
        timezone: "Asia/Kolkata",
      },
      isLast: false,
      isLastData: false,
      isLastDocument: false,
      isSurveyor: false,
      manualOdometerReading: "",
      source: "DIGITEDGE",
    };
    console.log(" FILE NAME   ", activityId + "_" + DocumentImageName + ext);
    RNFetchBlob.fetch(
      "POST",
      SELFSURVEYSERVICE_DOCUMENT_URL,
      {
        "Content-Type": "multipart/form-data",
      },
      [
        // element with property `filename` will be transformed into `file` in form data
        //file name is Original file name
        {
          name: "file",
          filename: activityId + "_" + DocumentImageName + ext,
          type: "image/jpg",
          data: uri,
        },
        { name: "documentData", data: JSON.stringify(documentUploadData) },
      ]
    )
      .uploadProgress({ interval: 10 }, (written, total) => {
        console.log("Written", written);
        console.log("total", total);
        console.log("uploaded", written / total);
        SetUploadState(written / total);
      })
      // listen to download progress event, every 10%
      .progress({ count: 10 }, (received, total) => {
        console.log("progress", received / total);
      })

      .then((response) => {
        Alert.alert("Upload completed");
        dispatch({
          type: "UPLOAD_IMAGE",
          payload: {
            uploadImageInfo: {
              upLoadStatus: uploadState,
            },
          },
        });
        console.log(
          "DATA FROM REDUX  >>> >> >",
          JSON.stringify(UploadImageData, null, "  ")
        );
      })
      .catch((err) => {
        console.log("******************** err: " + JSON.stringify(err));
      });
  };
  // create image with info
  const createImageWithName = async () => {
    if (title == VIDEO_TITLE || titleHn == VIDEO_TITLE_HN) {
      uploadImage(ImageData.uri);
      navigationFunc(ImageData.uri);
    } else {
      try {
        await captureRef(imageref, {
          format: "jpg",
          quality: 0.8,
        }).then(
          (uri) => {
            console.log(uri, " URI   >>");
            SetFinalImagePath(uri);
            if (title == SELFI_IMAGE || titleHn == SELFI_IMAGE_HN) {
              uploadImage(uri);
              navigationFunc(uri);
              reactotron.log("IN SELFI");
            } else {
              if (DocumentImageName === CHASSIS_NUMBER) {
                // call API for chassies number pose detection
                callChassisPoseDetection(uri);
              } else if (
                activityTypeId ==
                ACTIVITY_TYPE_MOTOR_2W_PRE_INSPECTION_SELF_SURVEY
              ) {
                // call APi for pose detection for 2W
                callPoseDetection(uri);
              } else if (
                activityTypeId ==
                  ACTIVITY_TYPE_MOTOR_PRE_INSPECTION_SELF_SURVEY ||
                activityTypeId ==
                  ACTIVITY_TYPE_MOTOR_4W_OD_NB_PRE_INSPECTION_SELF_SURVEY
              ) {
                // call API for 4W pose detection
                callPoseDetection(uri);
              }
            }
          },

          (error) => console.error("Oops, snapshot failed", error)
        );
      } catch (err) {}
    }
  };
  const callRetrayDataCount = () => {
    reTryDataLen = RetryImageData.retryImageInfo.length;
    console.log("REDUX LEN >", reTryDataLen);
  };
  return (
    <View style={styles.main}>
      {callRetrayDataCount()}
      <StatusBar hidden={true} />
      {/* Image View */}
      <View style={styles.imageView}>
        {/* Image */}
        {console.log(" IMAGE DATA   URI >> ")}
        {title == VIDEO_TITLE || title == VIDEO_TITLE_HN ? (
          <VideoPlayer
            video={{ uri: ImageData }}
            videoHeight={SCREEN_HEIGHT + 82}
            pauseOnPress={true}
            autoplay={true}
            disableSeek={true}
            // style={styles.productImageStyle}
          ></VideoPlayer>
        ) : (
          <ViewShot ref={imageref}>
            <View>
              <Image
                source={{ uri: ImageData.uri }}
                style={{
                  width: SCREEN_HEIGHT,
                  height: SCREEN_WIDTH,
                  resizeMode: "contain",
                  alignSelf: "center",
                }}
              />
              {/* info Strip */}
              <View style={styles.infoTextView}>
                <Text style={styles.infoText}>
                  {translate("Lat:") +
                    " " +
                    currentLatitude +
                    "  " +
                    translate("Lon:") +
                    "  " +
                    currentLongitude +
                    "  " +
                    "    |    " +
                    "Digit App" +
                    "    |    " +
                    dateAndTime}
                </Text>
              </View>
            </View>
          </ViewShot>
        )}
      </View>
      {/* Button View */}
      <View style={styles.buttonView}>
        {/* Retake View */}
        <View style={styles.retakeView}>
          <TouchableOpacity
            onPress={() => {
              navigation.navigate("Camera");
            }}
          >
            <Image
              source={retake}
              style={{ width: 18, height: 18, alignSelf: "center" }}
            />
            <Text style={styles.retakeText}>{translate("Retake")}</Text>
          </TouchableOpacity>
        </View>
        {/* Proceed View */}
        <View style={styles.proceedView}>
          <TouchableOpacity
            onPress={() => {
              createImageWithName();
              startCapturingSensorData();
            }}
          >
            <Image
              source={proceed}
              style={{ width: 49, height: 49, alignSelf: "center" }}
            />
            <Text style={styles.retakeText}>{translate("Proceed")}</Text>
          </TouchableOpacity>
        </View>
        {/* Blank View */}
        <View style={styles.blankView}></View>
      </View>
    </View>
  );
};
export default PreviewImage;

const styles = StyleSheet.create({
  main: {
    flex: 1,
    flexDirection: "row",
  },
  buttonView: {
    flex: 0.1,
    backgroundColor: "#000000",
  },
  imageView: {
    flex: 0.9,
  },
  retakeView: {
    flex: 0.333333,
    justifyContent: "center",
  },
  proceedView: {
    flex: 0.333333,
    justifyContent: "center",
  },
  blankView: {
    flex: 0.3333333,
  },
  retakeText: {
    fontFamily: "Muli-Regular",
    fontSize: 11,
    alignSelf: "center",
    marginTop: 5,
    color: "#FFFFFF",
  },
  infoTextView: {
    // flex:0.1,
    position: "absolute",
    width: SCREEN_HEIGHT,
    height: 48,
    backgroundColor: "#000000",
    opacity: 0.58,
    justifyContent: "center",
  },
  infoText: {
    fontFamily: "Muli-SemiBold",
    fontSize: 14,
    color: "#FFFFFF",
    marginLeft: 24,
    marginRight: 24,
  },
});
