import React, { useState, useEffect } from "react";
import { ScrollView, StyleSheet, Text, View, Image } from "react-native";
import {
  SCREEN_WIDTH,
  SCREEN_HEIGHT,
  VIDEO_TITLE_HN,
  VIDEO_TITLE,
  RC_BOOK,
  RC_BOOK_HN,
  ODOMETER,
  ODOMETER_HN,
} from "../common/Constants";
import VideoPlayer from "react-native-video-player";
import { translate } from "react-native-digitcamera/src/common/localize/i18n";
import WarningContent from "../common/WarningContent";
import { TextInput } from "../common/ThemedComponents";
import Carousel, { Pagination } from "react-native-snap-carousel";
import { PrimaryButton } from "../common/ThemedComponents";
import Database from "../../../../Components/common/Database";
import reactotron from "reactotron-react-native";
import Validation, { renderErrorMsge } from "../common/Validation";

const camera = require("../common/Image/camera.png");

const VechicleSurveyInstruction = ({ title, subTitle, points, image }) => {
  const [logoutModelVisible, setLogoutModelVisible] = useState(false);
  const [odometerReadings, setOdometerReadings] = useState(0);

  const rcBookData = [
    {
      name: "RCB1",
      image: "https://cdn.godigit.com/digitEdgeAssets/Motor/4W_PI_RCB1_V2.jpg",
      info: "RC FRONT SAMPLE",
    },
    {
      name: "RCB2",
      image: "https://cdn.godigit.com/digitEdgeAssets/Motor/4W_PI_RCB2_V2.jpg",
      info: "RC BACK SAMPLE",
    },
  ];
  let realm = Database.getRealm();
  var RealmData = realm.objects("SelectedLang");
  useEffect(() => {}, []);

  const _renderRcBookData = ({ item, index }) => {
    return (
      // Image View
      <View style={styles.imageView}>
        <View
          style={{
            flex: 0.95,
            justifyContent: "center",
            borderWidth: 1,
            borderColor: "#DDDDDD",
            borderRadius: 8,
          }}
        >
          <Image
            source={{ uri: item.image }}
            style={{
              width: SCREEN_WIDTH - 100,
              height: SCREEN_WIDTH / 1.47,
              resizeMode: "contain",
              alignSelf: "center",
            }}
          />
        </View>
        <View style={{ flex: 0.05 }}>
          <Text style={styles.sampleText}>{translate(item.info)}</Text>
        </View>
      </View>
    );
  };
  return (
    <ScrollView>
      {title == RC_BOOK || title == RC_BOOK_HN ? (
        <View>
          <Carousel
            data={rcBookData}
            renderItem={_renderRcBookData}
            sliderWidth={SCREEN_WIDTH}
            itemWidth={SCREEN_WIDTH}
            autoplay={true}
            loop={true}
          />
        </View>
      ) : (
        <View style={styles.main}>
          <View style={styles.vechicleImageView}>
            {title != VIDEO_TITLE || title != VIDEO_TITLE_HN ? (
              <Image
                source={{ uri: image }}
                style={{
                  width: SCREEN_WIDTH - 48,
                  height: SCREEN_WIDTH / 2,
                  alignSelf: "center",
                  resizeMode: "contain",
                }}
              />
            ) : (
              <VideoPlayer
                video={{ uri: image }}
                videoHeight={SCREEN_HEIGHT}
                videoWidth={SCREEN_WIDTH - 48}
                pauseOnPress={true}
                autoplay={true}
                disableSeek={true}
                style={styles.productImageStyle}
              ></VideoPlayer>
            )}
          </View>
          {/* Title View */}
          <View style={styles.titleView}>
            {title == VIDEO_TITLE || title == VIDEO_TITLE_HN ? (
              <Text
                style={{
                  fontFamily: "Muli-SemiBold",
                  fontSize: 12,
                  color: "#999999",
                  marginBottom: 29,
                }}
              >
                {translate("SAMPLE VIDEO")}
              </Text>
            ) : null}
            <Text style={styles.titleStyle}>{title}</Text>

            {/*  display only for odometer*/}
            {title == ODOMETER || title == ODOMETER_HN ? (
              <TextInput
                style={{ marginTop: 20 }}
                label={translate("Enter Odometer Reading (in Km)")}
                value={odometerReadings}
                onChangeText={(value) => {
                  setOdometerReadings(value);
                }}
                errorMsge={
                  Number(odometerReadings) < 1
                    ? renderErrorMsge({
                        type: "input",
                        text: "Odometer Reding",
                      })
                    : null
                }
              />
            ) : null}
          </View>
          {/* subTitle View */}
          <View style={styles.subTitleView}>
            <Text style={styles.subTitleText}>{subTitle}</Text>
          </View>
          {/* points View */}
          <View style={styles.pointsView}>
            {points?.map((item) => {
              return (
                <Text style={styles.pointsText}>
                  {"\u2022 "}
                  {RealmData[0]?.PreferredLanguage == "en"
                    ? item?.instructionData?.includes("Note:")
                      ? item?.instructionData
                          ?.split("Note:")[0]
                          ?.trim()
                          .split("<br>")
                          .join("")
                      : item?.instructionData?.split(".")?.join("")
                    : item?.instructionDataHn?.includes("नोट:")
                    ? item?.instructionDataHn?.split("नोट:")[0]?.trim()
                    : item?.instructionDataHn?.split(".")?.join("")}
                </Text>
              );
            })}
          </View>
          {points?.map((item) => {
            if (item?.instructionData?.includes("Note:")) {
              return (
                <WarningContent
                  style={{ marginTop: 16 }}
                  textContent={
                    "Note:" +
                    item?.instructionData
                      ?.split("Note:")[1]
                      ?.split(".")
                      ?.join("")
                  }
                ></WarningContent>
              );
            }
          })}
        </View>
      )}
    </ScrollView>
  );
};
const styles = StyleSheet.create({
  main: {
    backgroundColor: "#FFFFFF",
    flex: 1,
    marginRight: 24,
    marginLeft: 24,
    marginTop: 48,
  },
  vechicleImageView: {
    width: SCREEN_WIDTH - 48,
    height: (SCREEN_WIDTH - 48) / 1.7,
    justifyContent: "center",
    borderWidth: 1,
    borderColor: "#DDDDDD",
    borderRadius: 8,
  },
  titleStyle: {
    fontSize: 20,
    fontFamily: "Muli-Black",
  },
  pointsText: {
    fontSize: 16,
    fontFamily: "Muli-Regular",
    flex: 1,
    lineHeight: 32,
  },
  titleView: {
    marginTop: 48,
  },
  subTitleView: {
    marginTop: 16,
  },
  subTitleText: {
    fontFamily: "Muli-SemiBold",
    fontSize: 16,
  },
  pointsView: {
    marginTop: 18,
  },
  productImageStyle: {
    height: SCREEN_HEIGHT - 70,
  },
  imageView: {
    flex: 1,
    marginLeft: 24,
    marginRight: 24,
  },
  sampleText: {
    fontFamily: "Muli-SemiBold",
    fontSize: 12,
    color: "#999999",
    alignItems: "center",
    marginTop: 3,
  },
});
export default VechicleSurveyInstruction;
